; ; ; ; ; ; ; ; ; ; ; ; ; ; ; ; ; ; ; ; ; ; ; ; ; ; ; ; ; ; ; ; ; ; ; ; ; ; ; ; ;
;
;  RAD V3.3 (direct telnet)
;  AutoIt Version: 3.1.1.120 (beta)
; 
;  I got a kick to finally try out this AutoIt thing. Impressive!
;  There will be an AutoIt project No. 2, no doubt, but here's No. 1..
;
;  This is what I really wanted for RAD! *g*
;
;  All that sending keys to open windows is messy. If we can't have a *real*
;  scriptable *nix shell, then we should at least make the most of Windows' 
;  strengths; put everything in a nice GUI, and hide gubbins from the user.
;
;  I started all that voyager sendkeys business, sorry! In this version, all
;  the telnet action happens behind the scenes. the user only sees a cute GUI,
;  and NO risk of sending keys astray (the real issue with older versions).
;
;  Fortunately, the 205 doesn't implement a very complicated telnet negotiation.
;  In fact, sending telnet control codes seems to mess with it :/ anyway, it 
;  happily accepts any old data thrown down the wire. so let's do just that..
;
;  Thanks to RichE for pointing me at AutoIt. Win32 RAID for free! nize.
;  When it can cross-compile, it will be THE killer-app! <- PLEASE! *g*
;
;  ;o)
;  (or
;
;  at corz.org 2006
;
;  notes..
;       Rule 31 isn't deleted any more, some routers will disconnect if you
;		do that :/ If you want rule 31 deleted, simply add it to the list of
;       numbers inside "rules.txt" which is inside RAD's configuration folder, 
;       next to router.ini
;
;       If you want to compile this yourself, you will need to use a
;		recent AutoIt Beta. The release version has no TCP functions.
;
;  bugs.. 
;       not a bug, really, but if you pointed this thing at a *real* telnet 
;       server, it would likely report success, when so such thing occured!


#include <File.au3>
#include <String.au3>

; configuration..
$default_address = "192.168.1.1"
$default_name = "admin"
$default_pass = "admin"

; private configuration..
$my_version = " RAD V3" ; just for window titles
$my_self = "Voyager205" ; for titles and config folder name
$ini_file = "router.ini"

; I like small case!
$rules_file = "rules.rad" 
; in the absense of a $rules_file, it will be created, and these default rules will be inserted for deletion..
$delete_rules = "8,10,11,12,27,28,29,30,32,47,48,49,50,51,52,67,68,69,70,71,72,73,74,87,88,90,91,92,93,94,107,108"

; these non-escaping back-slashes are a bit disconcerting!
$data_path = @AppDataDir & "\" & $my_self
$ini_path = $data_path & "\" & $ini_file
$rules_path = $data_path & "\" & $rules_file

; default telnet port (usually 23)
$port_num = 23

; the configuration directory doesn't exist. let's create that..
if FileExists($data_path) == 0 Then
	DirCreate ($data_path)
	Msgbox(0, $my_self & " RAD Notice" , "my configuration data will be saved to.." & @CRLF & @CRLF & $data_path & @CRLF & @CRLF & "this is a one-time notice", 10)
endif

; check for ini file..
if FileExists($ini_path) then
	; grab the ini data and assign user variables..
	$default_address = iniread($ini_path, "router", "ip", $default_address)
	$default_name = iniread($ini_path, "router", "user", $default_name)
	$default_pass = iniread($ini_path, "router", "password", $default_pass)
endif

$address = InputBox($my_self & " login:",  "enter router's address..", $default_address, "",200, 80)
if @ERROR = 1 then rad_abort()
$username = InputBox($my_self & " login:", "your username..", $default_name, "",200, 80)
if @ERROR = 1 then rad_abort()
$password = InputBox($my_self & " login:", "your password..", $default_pass, "*",200, 80)
if @ERROR = 1 then rad_abort()

; in case they hit delete/backspace in a dialog..
if $address == "" then $address = $default_address
if $username == "" then $username = $default_name
if $password == "" then $password = $default_pass

; write current values to the ini file..
iniwrite($ini_path, "router", "ip", $address)
iniwrite($ini_path, "router", "user", $username)
iniwrite($ini_path, "router", "password", $password)

;check rules file exists, if not, create it..
if not FileExists($rules_path) then
	make_rules($rules_path, $delete_rules)
endif


; now the telnet..

; let's see if the host is up..
$ping_test = Ping($address, 500)

if $ping_test then ; ping is good

	; fire up TCP functions..
	TCPStartup()

	; attempt to connect a socket..
	$socket = TCPConnect($address, $port_num)

	if $socket <> -1 then ; socket is good

		SplashTextOn ($my_self & $my_version, @CRLF & "logging in.." , 133 , 66 ,-1 ,-1 , -1, "" , 13, 600)

		TCPSend ($socket, $username & @CRLF)
		sleep(300)
		TCPSend ($socket, $password & @CRLF)
		sleep(300)

		; simple check for login success..
		$response = TCPRecv($socket, 2048)
		if StringInStr($response, "failed") then
			SplashOff()
			MsgBox(64, $my_self, "Your login details are incorrect." & @CRLF & "Please double-check them.")
			exit
		endif

		; send our rules..
		SplashOff()
		ProgressOn ($my_self & $my_version, " sending rules to router..", " please wait a moment." , -1 , -1 , 16)

		if send_rules($socket, $rules_path) then
			ProgressOff()
			MsgBox(64, $my_self & $my_version, "Success! BT's automatic rules have been removed! Ha! ;o)", 10)
		else
			report_error()
		endif

	else
		report_error()
	endif

else
	MsgBox(48, $my_self & $my_version, "The router is not responding." & @CRLF & "Is it switched on?", 10)
endif

; thank-you, and good night.
TCPShutdown()




; here's the functions, out of the way..

; send some rules.
; I kept the login out of here to make it easier to alter this for other uses.
func send_rules ($socket, $rules_path)

	$rulescount = _FileCountLines($rules_path)
	$i = 1
	do
		$rulesread = filereadline($rules_path, $i)
		if StringIsDigit ($rulesread) and $rulesread <> "" then
			TCPSend ($socket, "delete ipf rule entry ruleid " & $rulesread & @CRLF)
			ProgressSet (((100 / $rulescount) * $i), "")
			sleep(250)
			$i = $i + 1
		else
			$i = $i + 1
		endif
	until $i = ($rulescount + 1)

	ProgressSet(-1,  " committing rules to memory")
	TCPSend ($socket, "commit" & @CRLF)
	sleep(2000)

	if @error then
		return 0
	else
		TCPSend ($socket, "exit" & @CRLF)
		sleep(300)
		return 1
	endif

endfunc


; put the default rule numbers in the file..
func make_rules ($file, $delete_rules)

	$rules = StringSplit($delete_rules, ",") ; create an array
	for $a = 1 to Ubound($rules) - 1
		FileWriteLine($file, $rules[$a])
	next

; alternatively, we could run StringReplace on the string, convert "," to @CRLF
; and write out the file once, but really, at this scale, it doesn't matter much.

endfunc


; see ya!
func rad_abort() 
	SplashTextOn ($my_self & $my_version, "ABORTED!" , 333 , 66 ,-1 ,-1 , -1, "" , 42, 1000)
	Sleep(1000)
	exit
endfunc


; oh oh!
func report_error() 
	SplashOff()
	ProgressOff()
	MsgBox(48, $my_self & $my_version, "an error occured" & @CRLF & "error: " & @error, 10)
	exit
endfunc
